from fhirclient.models.practitioner import Practitioner
from fhirclient.models.humanname import HumanName
from inspqcommun.fhir.visitors.base import BaseVisitor

class PractitionerVisitor(BaseVisitor):

    def __init__(self, fhir_resource=None) -> None:
        self.setFhirResource(fhir_resource=fhir_resource if fhir_resource else Practitioner())

    def getFhirResource(self) -> Practitioner:
        return super().getFhirResource()
    
    def get_id(self):
        return self.getFhirResource().id
    
    def get_family(self):
        if type(self.getFhirResource().name) is HumanName:
            return self.getFhirResource().name.family
        return None

    def get_given(self):
        if type(self.getFhirResource().name) is HumanName:
            return self.getFhirResource().name.given
        return None

    


from fhirclient.models.extension import Extension
from fhirclient.models.fhirdate import FHIRDate
from fhirclient.models.codeableconcept import CodeableConcept
from fhirclient.models.coding import Coding
from datetime import datetime

class BaseVisitor():
    DEFAULT_CODING_SYSTEM = 'http://www.santepublique.rtss.qc.ca/sipmi/fa/1.0.0/vocabulary'
    DEFAULT_CODING_VERSION = "1.0.0"

    __fhir_resource = None
    def __eq__(self, value: object) -> bool:
        return self.getFhirResource().as_json() == value.getFhirResource().as_json()
    
    def __contains__(self, value: object) -> bool:
        return self._is_struct_included(value.getFhirResource().as_json(), self.getFhirResource().as_json())
    
    def merge(self, value: object) -> None:
        if self.__fhir_resource is None:
            self.__fhir_resource = value.getFhirResource()
        else:
            self_json = self.getFhirResource().as_json()
            value_json = value.getFhirResource().as_json()
            self_json.update(value_json)
            new_fhir_resource = type(self.__fhir_resource)(jsondict=self_json)
            self.setFhirResource(new_fhir_resource)
            
    def getFhirResource(self):
        return self.__fhir_resource
    
    def setFhirResource(self, fhir_resource):
        self.__fhir_resource = fhir_resource

    def str_date_to_fhir_date(self, str_date):
        fhir_date  = FHIRDate()
        fhir_date.date = datetime.strptime(str_date, '%Y-%m-%d').date()
        return fhir_date

    def fhir_date_to_str_date(self, fhir_date):
        str_date = None
        if fhir_date is not None and fhir_date.date is not None:
            str_date = fhir_date.isostring
        return str_date

    def add_or_update_extension_to_extensions(self, extension, extensions):
        if extensions is None:
            extensions = [extension]
        else:
            ext_found = False
            for ext in extensions:
                if type(ext) is Extension:
                    if ext.url == extension.url:
                        extensions.remove(ext)
                        extensions.append(extension)
                        ext_found = True
                        break
                elif type(ext) is list:
                    for sub_ext in ext:
                        ext = self.add_or_update_extension_to_extensions(extension=sub_ext, extensions=ext)
            if not ext_found:
                extensions.append(extension)
        return extensions
    
    def _to_codeable_concept(self, code: str, display:str=None, coding_system: str=DEFAULT_CODING_SYSTEM, coding_version: str=DEFAULT_CODING_VERSION) -> CodeableConcept:
        codeableConcept = CodeableConcept()
        coding = Coding()
        coding.code = code
        if display:
            coding.display = display
        coding.system = coding_system
        coding.version = coding_version
        codeableConcept.coding = [coding]
        return codeableConcept
    
    def _get_coding_par_system(self, codeableConcept: CodeableConcept, coding_system:str=DEFAULT_CODING_SYSTEM, coding_version:str=DEFAULT_CODING_VERSION) -> Coding:
        coding: Coding
        coding_without_system = None
        for coding in codeableConcept.coding:
            if coding.system == coding_system and coding.version == coding_version:
                return coding
            elif coding.system is None:
                coding_without_system = coding 
        return coding_without_system
    
    def _creer_extension(self, url: str, **kwargs) -> Extension:
        extension = Extension()
        extension.url = url
        if "valueString" in kwargs:
            extension.valueString = kwargs["valueString"]
        elif "valueDate" in kwargs:
            extension.valueDate = kwargs["valueDate"]
        elif "valueCodeableConcept" in kwargs:
            extension.valueCodeableConcept = kwargs["valueCodeableConcept"]
        elif "valueInteger" in kwargs:
            extension.valueInteger = kwargs["valueInteger"]
        return extension
    
    def _is_struct_included(self, struct1, struct2, exclude=None):
        """
        This function compare if the first parameter structure is included in the second.
        The function use every elements of struct1 and validates they are present in the struct2 structure.
        The two structure does not need to be equals for that function to return true.
        Each elements are compared recursively.
        :param struct1:
            type:
                dict for the initial call, can be dict, list, bool, int or str for recursive calls
            description:
                reference structure
        :param struct2:
            type:
                dict for the initial call, can be dict, list, bool, int or str for recursive calls
            description:
                structure to compare with first parameter.
        :param exclude:
            type:
                list
            description:
                Key to exclude from the comparison.
            default: None
        :return:
            type:
                bool
            description:
                Return True if all element of dict 1 are present in dict 2, return false otherwise.
        """
        if isinstance(struct1, list) and isinstance(struct2, list):
            if not struct1 and not struct2:
                return True
            for item1 in struct1:
                if isinstance(item1, (list, dict)):
                    for item2 in struct2:
                        if self._is_struct_included(item1, item2, exclude):
                            break
                    else:
                        return False
                else:
                    if item1 not in struct2:
                        return False
            return True
        elif isinstance(struct1, dict) and isinstance(struct2, dict):
            if not struct1 and not struct2:
                return True
            try:
                for key in struct1:
                    if not (exclude and key in exclude):
                        if not self._is_struct_included(struct1[key], struct2[key], exclude):
                            return False
            except KeyError:
                return False
            return True
        else:
            return struct1 == struct2



from fhirclient.models.organization import Organization
from fhirclient.models.bundle import Bundle
from inspqcommun.fhir.clients.base_client import BaseClient
from requests import Response
import requests
import json
import logging

log = logging.getLogger(__name__)

class OrganizationClient(BaseClient):
    
    organization_endpoint = "{base_url}/Organization"
    search_endpoint = organization_endpoint + "/_search"
    organization_by_id_endpoint = organization_endpoint + "/{id}"

    def __init__(self, base_url=None, base_uri=None, token_header=None, validate_certs=True) -> None:
        super().__init__(base_url=base_url, base_uri=base_uri, token_header=token_header)
        self.validate_certs = validate_certs

    def search(self, name=None, identifier=None):
        params = {}
        if name is not None:
            params['name'] = name
        if identifier is not None:
            params['identifier'] = identifier
        response = requests.get(url=self.organization_endpoint.format(base_url=self.get_fhir_url()), params=params, headers=self.headers, verify=self.validate_certs)
        log.info("GET Organization?{} : {}".format("&".join([ key + "=" + params[key] for key in params ]), response.status_code))
        return response

    def get_by_id(self, org_id=None):
        response = None
        if org_id is not None:
            url = self.organization_by_id_endpoint.format(base_url=self.get_fhir_url(), id=org_id)
            response = self.__get_by_url(url=url)
        
        log.info("GET Organization/{} : {}".format(org_id, response.status_code if response.status_code else 404))
        return response
        
    def get_by_reference(self, reference=None):
        response = None
        if reference is not None:
            url = "{0}/{1}".format(self.get_fhir_url(), reference.reference)
            response = self.__get_by_url(url=url)
        
        log.info("GET Organization/{} : {}".format(reference.reference if reference is not None else '', response.status_code if response.status_code else 404))
        return response

    def get_all(self, page=0, page_size=10):
        page_params = {
            "page": page,
            "size": page_size
        }
        response = requests.get(url="{0}/_list".format(self.organization_endpoint.format(base_url=self.get_fhir_url())),
                                params=page_params,
                                headers=self.headers,
                                verify=self.validate_certs)
        log.info("GET Organization/_list : {}".format(response.status_code))
        return response
    
    def create(self, organization: Organization):
        response = requests.post(
            url=self.organization_endpoint.format(base_url=self.get_fhir_url()),
            data=json.dumps(organization.as_json()),
            headers=self.headers,
            verify=self.validate_certs)
        log.info("POST Organization : {}".format(response.status_code))
        return response

    def update(self, organization: Organization):
        response = requests.put(
            url=self.organization_by_id_endpoint.format(base_url=self.get_fhir_url(), id=organization.id),
            data=json.dumps(organization.as_json()),
            headers=self.headers,
            verify=self.validate_certs)
        log.info("PUT Organization/{} : {}".format(organization.id, response.status_code))
        return response

    def delete(self, organization: Organization = None, id = None):
        if organization or id:
            response = requests.delete(
                url=self.organization_by_id_endpoint.format(base_url=self.get_fhir_url(), id=organization.id if organization else id),
                headers=self.headers,
                verify=self.validate_certs)
            log.info("DELETE Organization/{} : {}".format(organization.id if organization else id, response.status_code))
            return response
        else:
            log.warn("DELETE Location - aucun id spécifié")
            return None

    def extract_organization_from_response(self, response: Response) -> Organization:
        if response.status_code == 200 or response.status_code == 201:
            content_dict = json.loads(response.content)
            return Organization(jsondict=content_dict)
        
    def extract_organization_bundle_from_response(self, response: Response) -> Bundle:
        if response.status_code == 200:
            content_dict = json.loads(response.content)
            return Bundle(jsondict=content_dict)
        return None
    
    def __get_by_url(self, url):        
        response = requests.get(
            url=url,
            headers=self.headers,
            verify=self.validate_certs
        )
        return response


from fhirclient.models.immunization import Immunization
from fhirclient.models.patient import Patient
from fhirclient.models.bundle import Bundle, BundleEntry,BundleEntryRequest
from fhirclient.models.reference import Reference
import requests
import json
import logging
import uuid
from requests import Response

log = logging.getLogger(__name__)

class BaseClient:

    base_headers = {
        "Content-type": "application/json+fhir"
    }

    def __init__(self, base_url=None, base_uri=None, token_header=None, validate_certs=True) -> None:
        self.base_url = base_url if base_url is not None else "http://localhost:14001"
        self.base_uri = base_uri if base_uri is not None else ""
        self.validate_certs = validate_certs
        self.set_headers(token_header=token_header)

    def set_headers(self, headers={}, token_header=None):
        new_headers = {**headers, **self.base_headers}
        if token_header is not None:
            if 'Content-Type' in token_header:
                del token_header['Content-Type']
            headers_with_auth = {**new_headers, **token_header}
            self.headers = headers_with_auth
        else:
            self.headers = new_headers
        return self.headers
    
    def get_fhir_url(self):
        return "{0}{1}".format(self.base_url, self.base_uri)
    
    def create_patient_with_immunization(self, patient:Patient, immunization:Immunization) -> Response:
        bundle = Bundle()
        bundle.type = 'transaction'
        entries = []
        patient_entry = BundleEntry()
        patient_entry.resource = patient
        patient_uuid = uuid.uuid4()
        patient_full_url = "urn:uuid:{uuid}".format(uuid=patient_uuid)
        patient_entry.fullUrl = patient_full_url
        patient_entry.request = BundleEntryRequest(jsondict={'method':'POST', 'url': 'Patient'})
        entries.append(patient_entry)
        immunization.patient = Reference(jsondict={'reference': patient_full_url})
        immunization_entry = BundleEntry()
        immunization_uuid = uuid.uuid4()
        immunization_full_url = "urn:uuid:{uuid}".format(uuid=immunization_uuid)
        immunization_entry.fullUrl = immunization_full_url
        immunization_entry.resource = immunization
        immunization_entry.request = BundleEntryRequest(jsondict={'method': 'POST', 'url': 'Immunization'})
        entries.append(immunization_entry)
        bundle.entry = entries
        data = json.dumps(bundle.as_json())
        response = requests.post(
            url="{base_url}/".format(base_url=self.get_fhir_url()),
            data=data,
            headers=self.headers,
            verify=self.validate_certs)
        log.info("POST Base URL : {}".format(response.status_code))
        return response




import os
import json
from json.decoder import JSONDecodeError
from typing import List
import logging

log = logging.getLogger(__name__)

class ChargeurFichiers:

    def __init__(self, chemin: str = None) -> None:
        if chemin:
            self.chemin: str = chemin if chemin.endswith("/") or chemin.endswith("\\") else chemin + "/"
        else:
            self.chemin:str = None

    def obtenir_noms_fichiers(self) -> List[str]:
        fichiers: List[str] = []
        for f in os.listdir(self.chemin):
            if os.path.isfile(self.chemin + f if self.chemin else f) and f.endswith('.json'):
                fichiers.append(self.chemin + f if self.chemin else f)
        log.info("Les fichiers du répertoire {} ont été obtenus".format(self.chemin if self.chemin else "racine au traitement"))
        return sorted(fichiers)
    
    def obtenir_contenu_fichier_par_nom(self, fichier) -> List[str]:
        f = open(fichier, encoding='utf-8')
        objet = None
        try:
            objet = json.loads(f.read())
            log.info("Le fichier {} a été décodé avec succèes".format(fichier))
        except JSONDecodeError:
            log.error("Le fichier {} n'a pas pu être décodé en format json. Il est invalide".format(fichier))
        f.close()
        return objet


from inspqcommun.fa.ressources.ressource import Ressource
from inspqcommun.fa.ressources.organisation import Organisation

class LieuVaccination(Ressource):

    def __init__(self, **kwargs) -> None:
        super().__init__("LieuVaccination")
        self.id : int = self._get_value_from_kwargs("id", **kwargs)
        self.nom : str = self._get_value_from_kwargs("nom", **kwargs)
        self.organisationParente : Organisation = self._get_value_from_kwargs("organisationParente", Organisation, **kwargs)


from fhirclient.models.location import Location
from fhirclient.models.fhirdate import FHIRDate
from fhirclient.models.period import Period
from fhirclient.models.extension import Extension
from fhirclient.models.identifier import Identifier
from fhirclient.models.address import Address
from fhirclient.models.reference import Reference
from fhirclient.models.contactpoint import ContactPoint
from inspqcommun.fhir.visitors.base import BaseVisitor

class LocationVisitor(BaseVisitor):
    
    RRSS_SYSTEM = 'https://pro.consultation.rrss.rtss.qc.ca'
    RRSS_MOT_CLE_EXTENSION_URL = 'http://www.santepublique.rtss.qc.ca/sipmi/rrss/1.0.0/extensions/#motscles'
    PERIOD_URL = "http://www.santepublique.rtss.qc.ca/sipmi/rrss/1.0.0/extensions/period"

    def __init__(self, fhir_resource=None) -> None:
        self.setFhirResource(fhir_resource=fhir_resource if fhir_resource else Location())

    def getFhirResource(self) -> Location:
        return super().getFhirResource()        

    def have_mot_cle(self, mot_cle=None):
        if mot_cle is not None and self.getFhirResource().extension is not None:
            for extension in self.getFhirResource().extension:
                if extension.url == self.RRSS_MOT_CLE_EXTENSION_URL and extension.valueString == mot_cle:
                    return True
        return False

    def set_name(self, name=None):
        self.getFhirResource().name = name

    def get_name(self):
        return self.getFhirResource().name

    def get_id(self):
        return self.getFhirResource().id

    def get_id_rrss(self):
        if self.getFhirResource().identifier is not None:
            for identifier in self.getFhirResource().identifier:
                if identifier.system == self.RRSS_SYSTEM:
                    return identifier.value
        return None

    def get_address_city(self):
        if self.getFhirResource().address:
            return self.getFhirResource().address.city
        return None

    def get_phones(self):
        phones = []
        if self.getFhirResource().telecom is None:
            return phones
        for telecom in self.getFhirResource().telecom:
            if telecom.system == "phone":
                phone = {}
                phone["value"] = telecom.value
                phone["use"] = telecom.use
                phones.append(phone)
        return phones
    
    def set_phones(self, phones:list[dict]=None):
        if phones is not None:
            telecoms = []
            for phone in phones:
                telecom = ContactPoint()

                telecom.system = phone["system"] if "system" in phone else "phone"
                telecom.value = phone["value"]
                telecom.use = phone["use"] if "use" in phone else "work"

                telecoms.append(telecom)
            self.getFhirResource().telecom = telecoms

    def get_effective_from(self):
        period_ext = self.__get_period_ext()
        if period_ext is not None and period_ext.valuePeriod is not None:
                return period_ext.valuePeriod.start
        return None

    def get_effective_to(self):
        period_ext = self.__get_period_ext()
        if period_ext is not None and period_ext.valuePeriod is not None:
                return period_ext.valuePeriod.end
        return None

    def set_effective_from(self, effective_from=None):
        fhir_effective_from = None
        if type(effective_from) is str:
            fhir_effective_from = FHIRDate(jsonval=effective_from)
        elif type(effective_from) is FHIRDate:
            fhir_effective_from = effective_from
        period_ext = self.__get_period_ext()
        if period_ext is None or period_ext.valuePeriod is None:
            period = Period()
        else:
            period = period_ext.valuePeriod
        period.start = fhir_effective_from
        self.__set_period_ext(period=period)

    def set_effective_to(self, effective_to=None):
        fhir_effective_to = None
        if type(effective_to) is str:
            fhir_effective_to = FHIRDate(jsonval=effective_to)
        elif type(effective_to) is FHIRDate:
            fhir_effective_to = effective_to
        period_ext = self.__get_period_ext()
        if period_ext is None or period_ext.valuePeriod is None:
            period = Period()
        else:
            period = period_ext.valuePeriod
        period.end = fhir_effective_to
        self.__set_period_ext(period=period)

    def is_active(self):
        return self.getFhirResource().status == "active"
    
    def __get_period_ext(self):
        if self.getFhirResource().extension is not None:
            for ext in self.getFhirResource().extension:
                if ext.url == self.PERIOD_URL:
                    return ext
        return None

    def __set_period_ext(self, period=None):
        if self.getFhirResource().extension is None:
            self.getFhirResource().extension = []
        period_ext = self.__get_period_ext()
        if period_ext is None:
            period_ext = Extension()
            period_ext.url = self.PERIOD_URL
            self.getFhirResource().extension.append(period_ext)
        period_ext.valuePeriod = period
        
    def set_id_rrss(self, id_rrss=None):
        if id_rrss is not None:
            identifier = Identifier()
            identifier.system = self.RRSS_SYSTEM
            identifier.value = id_rrss
            if self.get_id_rrss() is None:
                self.getFhirResource().identifier = []
                self.getFhirResource().identifier.append(identifier)
            else:
                for idx, id in enumerate(self.getFhirResource().identifier):
                    if id.system == self.RRSS_SYSTEM:
                        self.getFhirResource().identifier[idx] = identifier
                        break

    def set_address(self, address: Address=None, city: str=None, postal_code:str=None, country:str=None, state:str=None, line:str=None):
        if address is not None:
            self.getFhirResource().address = address
        else:
            if self.getFhirResource().address is None:
                self.getFhirResource().address = Address()
            self.getFhirResource().address.city = city
            if postal_code is not None:
                self.getFhirResource().address.postalCode = postal_code
            if country is not None:
                self.getFhirResource().address.country = country
            if state is not None:
                self.getFhirResource().address.state = state
            if line is not None:
                self.getFhirResource().address.line = [line]

    def get_address(self) -> Address:
        return self.getFhirResource().address
    
    def get_managing_organization(self) -> Reference:
        if self.getFhirResource().managingOrganization is not None:
            return self.getFhirResource().managingOrganization.reference
        return None

    def set_managing_organization(self, managing_organization:str=None):
        if managing_organization is not None:
            reference = Reference()
            reference.reference = managing_organization
            self.getFhirResource().managingOrganization = reference

    def get_description(self) -> str:
        return self.getFhirResource().description

    def set_description(self, description: str=None):
        self.getFhirResource().description = description


from fhirclient.models.medication import Medication, MedicationProduct
from inspqcommun.fhir.visitors.base import BaseVisitor

class MedicationVisitor(BaseVisitor):
    
    DEFAULT_QUANTITY_URL = 'http://www.santepublique.rtss.qc.ca/sipmi/fa/1.0.0/extensions/#medication/defaultquantity'
    DEFAULT_ROUTE_URL = 'http://www.santepublique.rtss.qc.ca/sipmi/fa/1.0.0/extensions/#medication/defaultroute'
    EXPIRED_URL = 'http://www.santepublique.rtss.qc.ca/sipmi/fa/1.0.0/extensions/#medication/expired'

    def __init__(self, fhir_resource=None) -> None:
        self.setFhirResource(fhir_resource=fhir_resource if fhir_resource else Medication())

    def getFhirResource(self) -> Medication:
        return super().getFhirResource()

    def get_batches(self):
        if type(self.getFhirResource().product) is MedicationProduct and type(self.getFhirResource().product.batch) is list:
            return self.getFhirResource().product.batch
        return []

    def get_default_quantity_for_batch(self, batch=None):
        quantity = None
        if batch is not None:
            for ext in batch.extension:
                if ext.url == self.DEFAULT_QUANTITY_URL:
                    quantity = ext.valueQuantity
        return quantity
    
    def get_default_route_for_batch(self, batch=None):
        route = None
        if batch is not None:
            for ext in batch.extension:
                if ext.url == self.DEFAULT_ROUTE_URL:
                    route = ext.valueCodeableConcept
        return route

    def is_batch_expired(self, batch=None):
        expired = False
        if batch is not None:
            for ext in batch.extension:
                if ext.url == self.EXPIRED_URL:
                    expired = ext.valueBoolean
        return expired
    
    def exists_lot_id_in_lots(self, lot_id: int) -> bool:
        if self.getFhirResource() and self.getFhirResource().product and self.getFhirResource().product.batch:
            for batch in self.getFhirResource().product.batch:
                if batch.id == str(lot_id):
                    return True
        return False

    def exists_lot_number_in_lots(self, lot_number: str) -> bool:
        if self.getFhirResource() and self.getFhirResource().product and self.getFhirResource().product.batch:
            for batch in self.getFhirResource().product.batch:
                if batch.lotNumber == lot_number:
                    return True
        return False


from datetime import date
from typing import List
from inspqcommun.fa.ressources.ressource import Ressource
from inspqcommun.fa.ressources.usager import Usager   
from inspqcommun.fa.ressources.vaccinateur import Vaccinateur
from inspqcommun.fa.ressources.lieu_vaccination import LieuVaccination 

class ActeVaccinal(Ressource):

    def __init__(self, **kwargs) -> None:
        self.usager : Usager = self._get_value_from_kwargs("usager", Usager, **kwargs)
        self.nomCommercial : str = self._get_value_from_kwargs("nomCommercial", **kwargs)
        self.agent : str = self._get_value_from_kwargs("agent", **kwargs)
        self.lot : ActeVaccinal.Lot = self._get_value_from_kwargs("lot", ActeVaccinal.Lot, **kwargs)
        self.dateAdministration : date = self._get_value_from_kwargs("dateAdministration", defaultValue=date.today(), **kwargs)
        self.quantiteAdministree : ActeVaccinal.QuantiteAdministree = self._get_value_from_kwargs("quantiteAdministree", ActeVaccinal.QuantiteAdministree, **kwargs)
        self.voieAdministration : str = self._get_value_from_kwargs("voieAdministration", **kwargs)
        self.siteAdministration : str = self._get_value_from_kwargs("siteAdministration", **kwargs)
        self.raisonAdministration : str = self._get_value_from_kwargs("raisonAdministration", **kwargs)
        self.vaccinateur : Vaccinateur = self._get_value_from_kwargs("vaccinateur", Vaccinateur, **kwargs)
        self.lieuVaccination : LieuVaccination = self._get_value_from_kwargs("lieuVaccination", LieuVaccination, **kwargs)
        self.commentaires : List[ActeVaccinal.Commentaire] = self._get_list_of_values_from_kwargs("commentaires", ActeVaccinal.Commentaire, **kwargs)
        
        super().__init__("ActeVaccinal")            

    class QuantiteAdministree(Ressource):

        def __init__(self, **kwargs) -> None:
            self.quantite : float = self._get_value_from_kwargs("quantite", **kwargs)
            self.unite : str = self._get_value_from_kwargs("unite", **kwargs)
    
    class Commentaire(Ressource):

        def __init__(self, **kwargs) -> None:
            self.auteur : str = self._get_value_from_kwargs("auteur", **kwargs) 
            self.date : date = self._get_value_from_kwargs("date", **kwargs)
            self.texte : str = self._get_value_from_kwargs("texte", **kwargs)
    
    class Lot(Ressource):

        def __init__(self, **kwargs) -> None:
            self.id : int = self._get_value_from_kwargs("id", **kwargs)
            self.numero : str = self._get_value_from_kwargs("numero", **kwargs)


import os
import socket
import jwt
import time
import uuid
from inspqcommun.identity.keycloak import KeycloakAPI, get_service_account_token, get_token
import ast

class KeycloakEnvironment():

    JWT_SECRET = 'secret'
    JWT_ALGO = 'HS256'
    
    def __init__(self,
                 defaultKeycloakPort=18081,
                 defaultAuthClientId="admin-cli",
                 defaultAuthClientSecret = None,
                 defaultProtocol="http",
                 defaultAuthRealm="master",
                 defaultAuthUser=None,
                 defaultAuthPassword=None,
                 defaultAdminAuthUser='admin',
                 defaultAdminAuthPassword='admin',
                 defaultAdminAuthRealm='master',
                 defaultValidateCert = False,
                 defaultKeycloakEnabled = True,
                 defaultBaseKeycloakUrl = "http://keycloak.url/auth",
                 ):
        self.defaultKeycloakPort = defaultKeycloakPort
        self.defaultAuthClientId = defaultAuthClientId
        self.defaultAuthClientSecret = defaultAuthClientSecret
        self.defaultProtocol = defaultProtocol
        self.defaultAuthRealm = defaultAuthRealm
        self.defaultAuthUser = defaultAuthUser
        self.defaultAuthPassword = defaultAuthPassword
        self.defaultAdminAuthUser = defaultAdminAuthUser
        self.defaultAdminAuthPassword = defaultAdminAuthPassword
        self.defaultValidateCert = defaultValidateCert
        self.defaultAdminAuthRealm = defaultAdminAuthRealm
        
        if 'KEYCLOAK_ENABLED' in os.environ:
            self.keycloak_enabled = bool(self.__str_to_bool(os.environ['KEYCLOAK_ENABLED']))
        else:
            self.keycloak_enabled = defaultKeycloakEnabled
        if 'KEYCLOAK_BASE_URL' in os.environ:
            self.keycloak_url = "{0}/auth".format(os.environ['KEYCLOAK_BASE_URL'])
        elif defaultBaseKeycloakUrl:
            self.keycloak_url = defaultBaseKeycloakUrl if defaultBaseKeycloakUrl.endswith('/auth') else defaultBaseKeycloakUrl + "/auth"
        else:
            self.keycloak_url = "{protocol}://{host}:{port}/auth".format(
                protocol=self.defaultProtocol,
                host=socket.getfqdn(),
                port=self.defaultKeycloakPort)
            print("URL Keycloak non specifie: Utilisation de l'URL par defaut")
        self.keycloak_auth_realm = os.environ['KEYCLOAK_AUTH_REALM'] if 'KEYCLOAK_AUTH_REALM' in os.environ else self.defaultAuthRealm
        self.keycloak_auth_client_id = os.environ['KEYCLOAK_AUTH_CLIENT_ID'] if 'KEYCLOAK_AUTH_CLIENT_ID' in os.environ else self.defaultAuthClientId
        self.keycloak_auth_client_secret = os.environ.get('KEYCLOAK_AUTH_CLIENT_SECRET', self.defaultAuthClientSecret)
        self.keycloak_auth_user = os.environ['KEYCLOAK_AUTH_USER'] if 'KEYCLOAK_AUTH_USER' in os.environ else self.defaultAuthUser
        self.keycloak_auth_password = os.environ['KEYCLOAK_AUTH_PASSWORD'] if 'KEYCLOAK_AUTH_PASSWORD' in os.environ else self.defaultAuthPassword
        self.keycloak_admin_auth_user = os.environ['KEYCLOAK_ADMIN_AUTH_USER'] if 'KEYCLOAK_ADMIN_AUTH_USER' in os.environ else self.defaultAdminAuthUser
        self.keycloak_admin_auth_password = os.environ['KEYCLOAK_ADMIN_AUTH_PASSWORD'] if 'KEYCLOAK_ADMIN_AUTH_PASSWORD' in os.environ else self.defaultAdminAuthPassword
        self.keycloak_admin_auth_realm = os.environ['KEYCLOAK_ADMIN_AUTH_REALM'] if 'KEYCLOAK_ADMIN_AUTH_REALM' in os.environ else self.defaultAdminAuthRealm
        self.validate_certs = bool(self.__str_to_bool(os.environ['VALIDATE_CERTS'])) if 'VALIDATE_CERTS' in os.environ else self.defaultValidateCert
        self.kc = self.init_keycloak_api()

    def init_keycloak_api(self):
        if self.keycloak_enabled and self.keycloak_admin_auth_user is not None and self.keycloak_admin_auth_password is not None:
            kcapi = KeycloakAPI(auth_keycloak_url=self.keycloak_url,
                         auth_client_id="admin-cli",
                         auth_username=self.keycloak_admin_auth_user,
                         auth_password=self.keycloak_admin_auth_password,
                         auth_realm=self.keycloak_admin_auth_realm,
                         auth_client_secret=None,
                         validate_certs=self.validate_certs)
        else:
            kcapi = None
        
        return kcapi
    
    def authenticate(self, client_roles=[], realm_roles=[], given_name=None, family_name=None, username=None, password=None):
        if self.keycloak_enabled:
            if not self.keycloak_auth_user or not self.keycloak_auth_password:
                return self.__authenticate_by_service_account()
            else:
                return self.__authenticate_by_username_password(username=username, password=password)
        else:
            return self.__get_test_access_token(
                client_roles=client_roles,
                realm_roles=realm_roles,
                given_name=given_name,
                family_name=family_name,
                preferred_username=username)

    def get_token_url(self):
        token_url = KeycloakAPI.format_token_url(baseurl=self.keycloak_url, realm=self.keycloak_auth_realm)
        return token_url

    def get_client_secret(self, client_id, realm=None):
        if realm is None:
            realm = self.keycloak_auth_realm
        if self.keycloak_enabled and (self.kc is not None or self.init_keycloak_api() is not None):
            keycloak_client = self.kc.get_client_by_clientid(client_id=client_id, realm=realm)
            keycloak_auth_client_secret = self.kc.get_client_secret_by_id(keycloak_client["id"], realm=realm)
            keycloak_auth_client_secret_value = keycloak_auth_client_secret['value'] if keycloak_auth_client_secret is not None and 'value' in keycloak_auth_client_secret else None
            return keycloak_auth_client_secret_value
        return None
    
    def __authenticate_by_service_account(self, client_id=None, client_realm=None):
        headers = {}
        client = client_id if client_id is not None else self.keycloak_auth_client_id
        realm = client_realm if client_realm is not None else self.keycloak_auth_realm

        if self.keycloak_enabled:
            if (self.keycloak_auth_client_secret is None or len(self.keycloak_auth_client_secret) == 0) and (self.kc is not None or self.init_keycloak_api() is not None):
                self.keycloak_auth_client_secret = self.get_client_secret(client_id=client, realm=realm)
            
            headers = get_service_account_token(
                base_url=self.keycloak_url,
                auth_realm=self.keycloak_auth_realm,
                client_id=client,
                client_secret=self.keycloak_auth_client_secret,
                validate_certs=self.validate_certs) if self.keycloak_auth_client_secret is not None else {}
        self.headers = headers
        return headers

    def __authenticate_by_username_password(self, username=None, password=None):
        headers = {}
        if self.keycloak_enabled:
            if (self.keycloak_auth_client_secret is None or len(self.keycloak_auth_client_secret) == 0) and (self.kc is not None or self.init_keycloak_api() is not None):
                self.keycloak_auth_client_secret = self.get_client_secret(client_id=self.keycloak_auth_client_id, realm=self.keycloak_auth_realm)
            if username is None:
                username = self.keycloak_auth_user
            if password is None:
                password = self.keycloak_auth_password
            headers = get_token(
                base_url=self.keycloak_url,
                validate_certs=self.validate_certs,
                auth_realm=self.keycloak_auth_realm,
                client_id=self.keycloak_auth_client_id,
                auth_username=username,
                auth_password=password, 
                client_secret=self.keycloak_auth_client_secret) if self.keycloak_auth_client_secret is not None else {}
        self.headers = headers
        return headers

    def __str_to_bool(self, value: str) -> bool:
        if value.upper() == "TRUE":
            value = "True"
        elif value.upper() == "FALSE":
            value = "False"
        bool_value = False
        try:
            bool_value = ast.literal_eval(value)
        except (SyntaxError, ValueError):
            return False
        return bool(bool_value)
    
    def __get_test_access_token(self, client_roles, realm_roles, given_name, family_name, preferred_username):
        if len(client_roles) == 0:
            client_roles = [
                "fa-saisie",
                "fa-utilisateur",
                "superuser"]
        if len(realm_roles) == 0:
            realm_roles = [
                "offline_access",
                "default-roles-msss",
                "uma_authorization"]
        if given_name is None:
            given_name = "Super"
        if family_name is None:
            family_name = "Permissions"
        if preferred_username is None:
            preferred_username = self.keycloak_auth_user

        decoded_access_token = {
            "exp": round(time.time()) + 300,
            "iat": round(time.time()),
            "auth_time": round(time.time()),
            "jti": str(uuid.uuid1()),
            "iss": "{}/realms/{}".format(self.keycloak_url, self.keycloak_auth_realm),
            "aud": self.keycloak_auth_client_id,
            "sub": str(uuid.uuid1()),
            "typ": "ID",
            "azp": self.keycloak_auth_client_id,
            "nonce": str(uuid.uuid1()),
            "session_state": str(uuid.uuid1()),
            "acr": "0",
            "email_verified": False,
            "groups": [
                "create-realm",
                "offline_access",
                "admin",
                "uma_authorization",
                "cluster-admin"
            ],
            "preferred_username": preferred_username,
            "allowed-origins": [
                "*"
            ],
            "realm_access": {
                "roles": realm_roles
            },
            "resource_access": {
                self.keycloak_auth_client_id: {
                    "roles": client_roles
                },
                "account": {
                "roles": [
                    "manage-account",
                    "manage-account-links",
                    "view-profile"
                ]
                }
        },
        "scope": "openid profile email",
        "sid": str(uuid.uuid1()),
        "name": "{given_name} {family_name}".format(given_name=given_name, family_name=family_name),
        "given_name": given_name,
        "family_name": family_name,
        "email": "nobody@inspq.qc.ca"
        }
        access_token = jwt.encode(decoded_access_token, self.JWT_SECRET, algorithm=self.JWT_ALGO, headers={
                                    'kid': 'NWBeViRdZb3-n0pBGu5YMJnaV1UMRMLjcvMOPJA2Gko', 'alg': self.JWT_ALGO, 'typ': 'jwt'})
        str_access_token = access_token.decode() if isinstance(access_token, (bytes,bytearray)) else access_token
        headers = {
            "Authorization": "bearer " + str_access_token
        }
        return headers




from inspqcommun.fhir.clients.base_client import BaseClient
from fhirclient.models.immunization import Immunization
from inspqcommun.fhir.visitors.parameters import ParametersVisitor
from requests import Response
import requests
import logging
import json

log = logging.getLogger(__name__)

class ImmunizationClient(BaseClient):

    immunization_endpoint = "{base_url}/Immunization"
    immunization_id_endpoint = "{base_url}/Immunization/{id}"
    immunization_id_override_endpoint = immunization_id_endpoint + '/$override'
    immunization_id_settoforecast_endpoint = immunization_id_endpoint + '/$settoforecast'
    immunization_id_delete_immunization_endpoint = immunization_id_endpoint + '/$delete-immunization'
    def __init__(self, base_url=None, base_uri=None, token_header=None, validate_certs=True) -> None:
        super().__init__(base_url=base_url, base_uri=base_uri, token_header=token_header)
        self.validate_certs = validate_certs

    def create(self, immunization: Immunization):
        immunization.meta = None
        response = requests.post(
            url=self.immunization_endpoint.format(base_url=self.get_fhir_url()),
            data=json.dumps(immunization.as_json()),
            headers=self.headers,
            verify=self.validate_certs)
        log.info("POST Immunization : {}".format(response.status_code))
        return response

    def get(self, immunization_id=None):
        response = requests.get(
            url=self.immunization_id_endpoint.format(base_url=self.get_fhir_url(),id=immunization_id),
            headers=self.headers,
            verify=self.validate_certs)
        log.info("GET Immunization : {}".format(response.status_code))
        return response
    
    def update(self, immunization: Immunization):
        response = requests.put(
            url=self.immunization_id_endpoint.format(base_url=self.get_fhir_url(), id=immunization.id),
            data=json.dumps(immunization.as_json()),
            headers=self.headers,
            verify=self.validate_certs)
        log.info("PUT Immunization : {}".format(response.status_code))
        return response

    def search(self, subject_id=None, code_agent=None, administration_date=None):
        params = {}
        if subject_id is not None:
            params['patient'] = subject_id
        if code_agent is not None:
            params['vaccine-code'] = code_agent
        if administration_date is not None:
            params['date'] = administration_date

        response = requests.get(
            url=self.immunization_endpoint.format(base_url=self.get_fhir_url()),
            params=params,
            headers=self.headers,
            verify=self.validate_certs
        )
        log.info("PUT Immunization : {}".format(response.status_code))
        return response

    def override(self, immunization: Immunization):
        parametre = ParametersVisitor()
        parametre.add_parameter(valeur=immunization, name='immunization',)
        res = parametre.getFhirResource().as_json()

        response = requests.post(
            url=self.immunization_id_override_endpoint.format(base_url=self.get_fhir_url(), id=immunization.id),
            data=json.dumps(res),
            headers=self.headers,
            verify=self.validate_certs)
        log.info("POST Immunization override : {}".format(response.status_code))
        return response

    def settoforecast(self, immunization: Immunization):
        parametre = ParametersVisitor()
        parametre.add_parameter(valeur=immunization, name='immunization',)
        res = parametre.getFhirResource().as_json()

        response = requests.post(
            url=self.immunization_id_settoforecast_endpoint.format(base_url=self.get_fhir_url(), id=immunization.id),
            data=json.dumps(res),
            headers=self.headers,
            verify=self.validate_certs)
        log.info("POST Immunization settoforecast : {}".format(response.status_code))
        return response

    def delete_immunization(self, immunization: Immunization):
        parametre = ParametersVisitor()
        parametre.add_parameter(valeur=immunization, name='immunization',)
        res = parametre.getFhirResource().as_json()

        response = requests.post(
            url=self.immunization_id_delete_immunization_endpoint.format(base_url=self.get_fhir_url(), id=immunization.id),
            data=json.dumps(res),
            headers=self.headers,
            verify=self.validate_certs)
        log.info("POST Immunization delete-immunization : {}".format(response.status_code))
        return response

    def extract_immunization_from_response(self, response: Response) -> Immunization:
        if response.status_code == 200 or response.status_code == 201:
            content = json.loads(response.content)
            return Immunization(jsondict=content)
        else:
            return None


from fhirclient.models.organization import Organization, OrganizationContact
from fhirclient.models.fhirdate import FHIRDate
from fhirclient.models.period import Period
from fhirclient.models.extension import Extension
from fhirclient.models.meta import Meta
from fhirclient.models.address import Address
from fhirclient.models.reference import Reference
from fhirclient.models.identifier import Identifier
from fhirclient.models.contactpoint import ContactPoint

from inspqcommun.fhir.visitors.base import BaseVisitor

class OrganizationVisitor(BaseVisitor):
    
    RRSS_SYSTEM = 'https://pro.consultation.rrss.rtss.qc.ca'
    HORS_RRSS_SYSTEM = 'https://horsrrss.santepublique.rtss.qc.ca'
    RRSS_MOT_CLE_EXTENSION_URL = 'http://www.santepublique.rtss.qc.ca/sipmi/rrss/1.0.0/extensions/#motscles'
    PERIOD_URL = "http://www.santepublique.rtss.qc.ca/sipmi/rrss/1.0.0/extensions/period"

    def __init__(self, fhir_resource=None) -> None:
        self.setFhirResource(fhir_resource=fhir_resource if fhir_resource else Organization())

    def getFhirResource(self) -> Organization:
        return super().getFhirResource()  

    def get_id(self) -> str:
        return self.getFhirResource().id

    def set_version_id(self, versionId: int):
        if not self.getFhirResource().meta:
            self.getFhirResource().meta = Meta()
        self.getFhirResource().meta.versionId = versionId

    def get_version_id(self) -> int:
        if self.getFhirResource().meta:
            return self.getFhirResource().meta.versionId
        return None

    def get_identifier_by_system(self, system:str) -> str:
        if self.getFhirResource().identifier is not None:
            for identifier in self.getFhirResource().identifier:
                if identifier.system == system:
                    return identifier.value
        return None
    
    def get_id_rrss(self) -> str:
        return self.get_identifier_by_system(self.RRSS_SYSTEM)
    
    def get_id_hors_rrss(self) -> str:
        return self.get_identifier_by_system(self.HORS_RRSS_SYSTEM)
    
    def set_identifier_by_system(self, system:str, value:str):
        if self.getFhirResource().identifier is None:
            self.getFhirResource().identifier = []
        for identifier in self.getFhirResource().identifier:
            if identifier.system == system:
                identifier.value = value
                return
        identifier = Identifier()
        identifier.system = system
        identifier.value = value
        self.getFhirResource().identifier.append(identifier)

    def set_id_rrss(self, id_rrss:str):
        self.set_identifier_by_system(self.RRSS_SYSTEM, id_rrss)

    def set_id_hors_rrss(self, hors_rrss_id:str):
        self.set_identifier_by_system(self.HORS_RRSS_SYSTEM, hors_rrss_id)

    def set_name(self, name:str=None):
        self.getFhirResource().name = name

    def get_name(self) -> str:
        return self.getFhirResource().name

    def have_mot_cle(self, mot_cle=None) -> bool:
        if mot_cle is not None and self.getFhirResource().extension is not None:
            for extension in self.getFhirResource().extension:
                if extension.url == self.RRSS_MOT_CLE_EXTENSION_URL and extension.valueString == mot_cle:
                    return True
        return False

    def get_effective_from(self) -> FHIRDate:
        period_ext = self.__get_period_ext()
        if period_ext is not None and period_ext.valuePeriod is not None:
                return period_ext.valuePeriod.start
        return None

    def get_effective_to(self) -> FHIRDate:
        period_ext = self.__get_period_ext()
        if period_ext is not None and period_ext.valuePeriod is not None:
                return period_ext.valuePeriod.end
        return None

    def set_effective_from(self, effective_from=None):
        fhir_effective_from = None
        if type(effective_from) is str:
            fhir_effective_from = FHIRDate(jsonval=effective_from)
        elif type(effective_from) is FHIRDate:
            fhir_effective_from = effective_from
        period_ext = self.__get_period_ext()
        if period_ext is None or period_ext.valuePeriod is None:
            period = Period()
        else:
            period = period_ext.valuePeriod
        period.start = fhir_effective_from
        self.__set_period_ext(period=period)

    def set_effective_to(self, effective_to=None):
        fhir_effective_to = None
        if type(effective_to) is str:
            fhir_effective_to = FHIRDate(jsonval=effective_to)
        elif type(effective_to) is FHIRDate:
            fhir_effective_to = effective_to
        period_ext = self.__get_period_ext()
        if period_ext is None or period_ext.valuePeriod is None:
            period = Period()
        else:
            period = period_ext.valuePeriod
        period.end = fhir_effective_to
        self.__set_period_ext(period=period)

    def get_telecoms(self) -> list[ContactPoint]:
        return self.getFhirResource().telecom
    
    def set_telecoms(self, phones:list[ContactPoint]=None):
        self.getFhirResource().telecom = phones

    def get_phones(self) -> list[dict]:
        phones = []
        if self.getFhirResource().telecom is None:
            return phones
        for telecom in self.getFhirResource().telecom:
            if telecom.system == "phone":
                phone = {}
                phone["value"] = telecom.value
                phone["use"] = telecom.use
                phones.append(phone)
        return phones
    
    def set_phones(self, phones:list[dict]=None):
        if phones is not None:
            telecoms = []
            for phone in phones:
                telecom = ContactPoint()

                telecom.system = phone["system"] if "system" in phone else "phone"
                telecom.value = phone["value"]
                telecom.use = phone["use"] if "use" in phone else "work"

                telecoms.append(telecom)
            self.set_telecoms(telecoms)

    def add_telecom(self, phone:ContactPoint):
        if self.getFhirResource().telecom is None:
            self.getFhirResource().telecom = []
        self.getFhirResource().telecom.append(phone)

    def add_phone_by_fields(self, phone:dict):
        telecom = ContactPoint()
        telecom.system = phone["system"] if "system" in phone else "phone"
        telecom.value = phone["value"]
        telecom.use = phone["use"] if "use" in phone else "work"
        self.add_telecom(telecom)   

    def get_contact_telecoms(self) -> list[ContactPoint]:
        if self.getFhirResource().contact is not None and len(self.getFhirResource().contact) > 0:
            return self.getFhirResource().contact[0].telecom
        return []

    def set_contact_telecoms(self, telecoms: list[ContactPoint] = None):
        if self.getFhirResource().contact is None:
            self.getFhirResource().contact = [OrganizationContact()]
        elif len(self.getFhirResource().contact) == 0:
            self.getFhirResource().contact.append(OrganizationContact())
        self.getFhirResource().contact[0].telecom = telecoms

    def get_contacts_phones(self) -> list[dict]:
        phones = []
        if self.getFhirResource().contact is None:
            return phones
        for contact in self.getFhirResource().contact:
            for telecom in contact.telecom:
                if telecom.system == "phone":
                    phone = {}
                    phone["value"] = telecom.value
                    phone["use"] = telecom.use
                    phones.append(phone)
        return phones
    
    def get_contact(self) -> list[OrganizationContact]:
        return self.getFhirResource().contact

    def set_contact(self, contact: list[OrganizationContact] = None):
        self.getFhirResource().contact = contact

    def set_contacts_phones(self, phones: list[dict] = None):
        if phones is not None:
            if self.get_contact() is None:
                self.set_contact([OrganizationContact()])
            contact = self.get_contact()

            if contact[0].telecom is None:
                contact[0].telecom = []
            
            for phone in phones:
                telecom = ContactPoint()
                telecom.system = phone["system"] if "system" in phone else "phone"
                telecom.value = phone["value"]
                telecom.use = phone["use"] if "use" in phone else "work"

                contact[0].telecom.append(telecom)

    def add_contact_phone(self, phone: dict):
        contact = None
        if self.get_contact() is None:
            self.set_contact([OrganizationContact()])
        elif len(self.get.get_contact()) == 0:
            self.get_contact.contact().append(OrganizationContact())
        contact = self.get_contact()[0]

        telecom = ContactPoint()
        telecom.system = phone["system"] if "system" in phone else "phone"
        telecom.value = phone["value"]
        telecom.use = phone["use"] if "use" in phone else "work"
        if contact.telecom is None:
            contact.telecom = []
        contact.telecom.append(telecom)

    def set_contact_purpose_by_fields(self, code: str = None, display: str = None, system: str = None, version: str = None):
        if self.get_contact() is None:
            self.set_contact([OrganizationContact()])
        elif len(self.get.get_contact()) == 0:
            self.get_contact.contact().append(OrganizationContact())
        contact = self.get_contact()[0]
        purpose = self._to_codeable_concept(code=code, display=display, coding_system=system, coding_version=version)
        contact.purpose = purpose

    def get_contact_address(self) -> Address:
        if self.get_contact() is not None and len(self.get_contact()) > 0:
            return self.get_contact()[0].address
        return None

    def set_contact_address(self, address: Address = None):
        if self.get_contact() is None:
            self.set_contact([OrganizationContact()])
        elif len(self.get_contact()) == 0:
            self.get_contact().append(OrganizationContact())
        self.get_contact()[0].address = address

    def set_contact_address_by_fields(self, line:list=None, city:str=None, state:str=None, postal_code:str=None, country:str=None, type:str='physical', use:str='work'):
        address = Address()
        address.line = line
        address.city = city
        address.state = state
        address.postalCode = postal_code
        address.country = country
        address.type = type
        address.use = use
        self.set_contact_address(address)

    def is_active(self) -> bool:
        return self.getFhirResource().active
    
    def set_active(self, active: bool=True):
        self.getFhirResource().active = active

    def __get_period_ext(self) -> Extension:
        if self.getFhirResource().extension is not None:
            for ext in self.getFhirResource().extension:
                if ext.url == self.PERIOD_URL:
                    return ext
        return None

    def __set_period_ext(self, period:Period=None):
        if self.getFhirResource().extension is None:
            self.getFhirResource().extension = []
        period_ext = self.__get_period_ext()
        if period_ext is None:
            period_ext = Extension()
            period_ext.url = self.PERIOD_URL
            self.getFhirResource().extension.append(period_ext)
        period_ext.valuePeriod = period

    def get_address(self) -> Address:
        return self.getFhirResource().address

    def set_address(self, address:Address=None):
        self.getFhirResource().address = address

    def set_address_by_fields(self, line:list=None, city:str=None, state:str=None, postal_code:str=None, country:str=None):
        address = Address()
        address.line = line
        address.city = city
        address.state = state
        address.postalCode = postal_code
        address.country = country
        self.set_address(address)

    def get_part_of(self) -> Reference:
        return self.getFhirResource().partOf

    def set_part_of(self, part_of: Reference=None):
        self.getFhirResource().partOf = part_of

    def set_part_of_by_fields(self, part_of_id: str=None, part_of_display: str=None, part_of_ref: str=None):
        part_of_reference = Reference()
        part_of_reference.reference = f"Organization/{part_of_ref}"
        part_of_reference.display = part_of_display
        part_of_reference.id = part_of_id
        self.set_part_of(part_of_reference)

    def get_mots_cles(self) -> list[Extension]:
        mots_cles = []
        if self.getFhirResource().extension is not None:
            for ext in self.getFhirResource().extension:
                if ext.url == self.RRSS_MOT_CLE_EXTENSION_URL:
                    mots_cles.append(ext.valueString)
        return mots_cles


from fhirclient.models.location import Location
from fhirclient.models.bundle import Bundle
from inspqcommun.fhir.clients.base_client import BaseClient
from requests import Response
import requests
import json
import logging

log = logging.getLogger(__name__)

class LocationClient(BaseClient):

    location_endpoint = "{base_url}/Location"
    location_by_id_endpoint = location_endpoint + "/{id}"

    def __init__(self, base_url=None, base_uri=None, token_header=None, validate_certs=True) -> None:
        super().__init__(base_url=base_url, base_uri=base_uri, token_header=token_header)
        self.validate_certs = validate_certs

    def search(self, name=None, address_city=None, identifier=None):
        params = {}
        if name is not None:
            params['name'] = name
        if identifier is not None:
            params['identifier'] = identifier
        if address_city is not None:
            params['address-city'] = address_city
        response = requests.get(
            url=self.location_endpoint.format(base_url=self.get_fhir_url()),
            params=params,
            headers=self.headers,
            verify=self.validate_certs)
        log.info("GET Location?{} : {}".format("&".join([ key + "=" + params[key] for key in params ]), response.status_code))
        return response

    def get_by_id(self, location_id=None):
        response = None
        if location_id is not None:
            response = requests.get(
                url=self.location_by_id_endpoint.format(base_url=self.get_fhir_url(), id=location_id),
                headers=self.headers,
                verify=self.validate_certs)
        
        log.info("GET Location/{} : {}".format(location_id, response.status_code if response.status_code else 404))
        return response

    def get_all(self, page=0, page_size=10):
        page_params = {
            "page": page,
            "size": page_size
        }
        response = requests.get(url="{0}/_list".format(self.location_endpoint.format(base_url=self.get_fhir_url())),
                                params=page_params,
                                headers=self.headers,
                                verify=self.validate_certs)
        log.info("GET Location/_list : {}".format(response.status_code))
        return response
    
    def create(self, location: Location):
        response =  requests.post(
            url=self.location_endpoint.format(base_url=self.get_fhir_url()),
            data=json.dumps(location.as_json()),
            headers=self.headers,
            verify=self.validate_certs)
        log.info("POST Location : {}".format(response.status_code))
        return response

    def update(self, location: Location):
        response = requests.put(
            url=self.location_by_id_endpoint.format(base_url=self.get_fhir_url(), id=location.id),
            data=json.dumps(location.as_json()),
            headers=self.headers,
            verify=self.validate_certs)
        log.info("PUT Location/{} : {}".format(location.id, response.status_code))
        return response

    def delete(self, location: Location = None, id: int = None):
        if location or id:
            response = requests.delete(
                url=self.location_by_id_endpoint.format(base_url=self.get_fhir_url(), id=location.id if location else id),
                headers=self.headers,
                verify=self.validate_certs)
            log.info("DELETE Location/{} : {}".format(location.id if location else id, response.status_code))
            return response
        else:
            log.warn("DELETE Location - aucun id spécifié")
            return None
    
    def extract_location_bundle_from_response(self, response: Response) -> Bundle:
        if response.status_code == 200:
            content_dict = json.loads(response.content)
            return Bundle(jsondict=content_dict)
        else:
            return None
    
    def extract_location_from_response(self, response: Response) -> Location:
        if response.status_code == 200 or response.status_code == 201:
            content = json.loads(response.content)
            if "entry" in content and len(content["entry"]) > 0:
                return Location(jsondict=content["entry"][0]["resource"])
            elif "resourceType" in content and content["resourceType"] == "Location":
                return Location(jsondict=content)
        return None